
# q-expansion of newform 1920.2.a.x, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1920
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [1], [4], [-6], [-4], [4], [8], [0], [2], [2], [4], [6], [12], [0], [-14], [6], [-6], [4], [8], [-2], [-6], [-12], [6], [-14], [-10], [16], [8], [-18], [-12], [-16], [2], [-8], [8], [18], [6], [0], [12], [0], [6], [-18], [-18], [-12], [14], [2], [2], [8], [16], [-8], [-22], [-8], [12], [-18], [-18], [4], [-16], [-30], [-22], [8], [6], [4], [18], [4], [-12], [6], [-14], [12], [30], [-24], [2], [-16], [-20], [-4], [32], [-12], [-8], [6], [4], [2], [-18], [-34], [10], [-28], [-18], [10], [0], [42], [-22], [10], [-36], [-12], [0], [16], [26], [4], [32], [-26], [-14], [-20], [-10], [20], [30], [32], [18], [-28], [-6], [16], [-4], [-12], [10], [-28], [-4], [-24], [24], [-38], [-18], [12], [0], [14], [10], [-38], [-18], [6], [20], [16], [38], [-38], [4], [32], [40], [44], [-8], [2], [48], [2], [10], [18], [-12], [-18], [6], [-52], [-54], [8], [-20], [-2], [36], [-28], [4], [28], [48], [-32], [-58], [-20], [32], [44], [-8], [50], [34], [-46], [14], [52], [-8], [16], [30], [4], [-56], [30], [-24], [2], [30], [6], [62], [0], [22], [-46], [14], [-64], [18], [-20], [-42], [-24], [38], [32], [52], [-16], [-58], [-28], [-36], [-2], [-52], [-34], [12], [-32], [18], [-40], [36], [-18], [60], [60], [-8], [30], [54], [-36], [-50], [-14], [42], [22], [-48], [-18], [48], [-18], [26], [-16], [12], [-32], [-14], [12], [18], [32], [6], [50], [-22], [6], [20], [4], [-74], [36], [-20], [-12], [-18], [-44], [52], [-6], [30], [4], [0], [6], [6], [2], [48], [8], [20], [24], [-38], [-48], [-16], [-28], [54], [4], [-64], [48], [-62], [8], [-10], [-14], [42], [6], [-52], [2], [-26], [-8], [44], [50], [28], [48], [28], [78], [-6], [-76], [62], [62], [28], [38], [10], [-58], [24], [56], [-22], [-74], [-46], [72], [-38], [-56], [-2], [-28], [-68], [58], [54], [14], [18], [78], [56], [24], [14], [4], [26], [46], [42], [18], [-44], [6], [-58], [54], [-48], [32], [-78], [52], [42], [72], [0], [48], [10], [-22], [-52], [48], [-30], [-66], [8], [22], [74], [-20], [38], [-42], [16], [-28], [50], [28], [20], [-24], [-34], [2], [-18], [-66], [72], [-32], [-4], [-2], [-72], [50], [0], [-32], [-12], [6], [50], [-6], [-38], [-38], [-12], [80], [-22], [-92], [42], [-10], [16], [10], [48], [72], [-58], [-72], [-88], [-32], [-78], [24], [-54], [-60], [-38], [-62], [60], [-10], [-38], [-4], [-56], [-86], [-66], [8], [-14], [-16], [50], [50], [10], [-54], [-24], [-20], [80], [-68], [40], [10], [8], [60], [-16], [-14], [78], [-26], [-28], [88], [58], [2], [30], [48], [-42], [94], [72], [-32], [16], [-6], [10], [104], [50], [20], [-42], [26], [-10], [-4], [56], [-26], [-94], [88], [68], [-72], [-64], [-6], [48], [40], [6], [22], [18], [60], [-94], [80], [60]]
